@@ -1,4 +1,4 @@
-from typing import Dict
+from __future__ import annotations
 
 from .base import Always, Filter, FilterOrBool, Never
 
@@ -12,7 +12,7 @@ _always = Always()
 _never = Never()
 
 
-_bool_to_filter: Dict[bool, Filter] = {
+_bool_to_filter: dict[bool, Filter] = {
     True: _always,
     False: _never,
 }
@@ -29,7 +29,7 @@ def to_filter(bool_or_filter: FilterOrBool) -> Filter:
     if isinstance(bool_or_filter, Filter):
         return bool_or_filter
 
-    raise TypeError("Expecting a bool or a Filter instance. Got %r" % bool_or_filter)
+    raise TypeError(f"Expecting a bool or a Filter instance. Got {bool_or_filter!r}")
 
 
 def is_true(value: FilterOrBool) -> bool:
