@@ -6,21 +6,13 @@ A lot of thanks, regarding outputting of colors, goes to the Pygments project:
 everything has been highly optimized.)
 http://pygments.org/
 """
+
+from __future__ import annotations
+
 import io
 import os
 import sys
-from typing import (
-    Callable,
-    Dict,
-    Hashable,
-    Iterable,
-    List,
-    Optional,
-    Sequence,
-    Set,
-    TextIO,
-    Tuple,
-)
+from typing import Callable, Dict, Hashable, Iterable, Sequence, TextIO, Tuple
 
 from prompt_toolkit.cursor_shapes import CursorShape
 from prompt_toolkit.data_structures import Size
@@ -159,10 +151,10 @@ class _16ColorCache:
 
     def __init__(self, bg: bool = False) -> None:
         self.bg = bg
-        self._cache: Dict[Hashable, _ColorCodeAndName] = {}
+        self._cache: dict[Hashable, _ColorCodeAndName] = {}
 
     def get_code(
-        self, value: Tuple[int, int, int], exclude: Sequence[str] = ()
+        self, value: tuple[int, int, int], exclude: Sequence[str] = ()
     ) -> _ColorCodeAndName:
         """
         Return a (ansi_code, ansi_name) tuple. (E.g. ``(44, 'ansiblue')``.) for
@@ -177,9 +169,8 @@ class _16ColorCache:
         return cache[key]
 
     def _get(
-        self, value: Tuple[int, int, int], exclude: Sequence[str] = ()
+        self, value: tuple[int, int, int], exclude: Sequence[str] = ()
     ) -> _ColorCodeAndName:
-
         r, g, b = value
         match = _get_closest_ansi_color(r, g, b, exclude=exclude)
 
@@ -199,7 +190,7 @@ class _256ColorCache(Dict[Tuple[int, int, int], int]):
 
     def __init__(self) -> None:
         # Build color table.
-        colors: List[Tuple[int, int, int]] = []
+        colors: list[tuple[int, int, int]] = []
 
         # colors 0..15: 16 basic colors
         colors.append((0x00, 0x00, 0x00))  # 0
@@ -235,7 +226,7 @@ class _256ColorCache(Dict[Tuple[int, int, int], int]):
 
         self.colors = colors
 
-    def __missing__(self, value: Tuple[int, int, int]) -> int:
+    def __missing__(self, value: tuple[int, int, int]) -> int:
         r, g, b = value
 
         # Find closest color.
@@ -266,7 +257,7 @@ _256_colors = _256ColorCache()
 class _EscapeCodeCache(Dict[Attrs, str]):
     """
     Cache for VT100 escape codes. It maps
-    (fgcolor, bgcolor, bold, underline, strike, reverse) tuples to VT100
+    (fgcolor, bgcolor, bold, underline, strike, italic, blink, reverse, hidden, dim) tuples to VT100
     escape sequences.
 
     :param true_color: When True, use 24bit colors instead of 256 colors.
@@ -286,13 +277,16 @@ class _EscapeCodeCache(Dict[Attrs, str]):
             blink,
             reverse,
             hidden,
+            dim,
         ) = attrs
-        parts: List[str] = []
+        parts: list[str] = []
 
         parts.extend(self._colors_to_code(fgcolor or "", bgcolor or ""))
 
         if bold:
             parts.append("1")
+        if dim:
+            parts.append("2")
         if italic:
             parts.append("3")
         if blink:
@@ -314,7 +308,7 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         self[attrs] = result
         return result
 
-    def _color_name_to_rgb(self, color: str) -> Tuple[int, int, int]:
+    def _color_name_to_rgb(self, color: str) -> tuple[int, int, int]:
         "Turn 'ffffff', into (0xff, 0xff, 0xff)."
         try:
             rgb = int(color, 16)
@@ -335,7 +329,7 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         # same. (Unless they were explicitly defined to be the same color.)
         fg_ansi = ""
 
-        def get(color: str, bg: bool) -> List[int]:
+        def get(color: str, bg: bool) -> list[int]:
             nonlocal fg_ansi
 
             table = BG_ANSI_COLORS if bg else FG_ANSI_COLORS
@@ -377,14 +371,14 @@ class _EscapeCodeCache(Dict[Attrs, str]):
                 else:
                     return [(48 if bg else 38), 5, _256_colors[rgb]]
 
-        result: List[int] = []
+        result: list[int] = []
         result.extend(get(fg_color, False))
         result.extend(get(bg_color, True))
 
         return map(str, result)
 
 
-def _get_size(fileno: int) -> Tuple[int, int]:
+def _get_size(fileno: int) -> tuple[int, int]:
     """
     Get the size of this pseudo terminal.
 
@@ -400,32 +394,40 @@ class Vt100_Output(Output):
     :param get_size: A callable which returns the `Size` of the output terminal.
     :param stdout: Any object with has a `write` and `flush` method + an 'encoding' property.
     :param term: The terminal environment variable. (xterm, xterm-256color, linux, ...)
+    :param enable_cpr: When `True` (the default), send "cursor position
+        request" escape sequences to the output in order to detect the cursor
+        position. That way, we can properly determine how much space there is
+        available for the UI (especially for drop down menus) to render. The
+        `Renderer` will still try to figure out whether the current terminal
+        does respond to CPR escapes. When `False`, never attempt to send CPR
+        requests.
     """
 
     # For the error messages. Only display "Output is not a terminal" once per
     # file descriptor.
-    _fds_not_a_terminal: Set[int] = set()
+    _fds_not_a_terminal: set[int] = set()
 
     def __init__(
         self,
         stdout: TextIO,
         get_size: Callable[[], Size],
-        term: Optional[str] = None,
-        default_color_depth: Optional[ColorDepth] = None,
+        term: str | None = None,
+        default_color_depth: ColorDepth | None = None,
         enable_bell: bool = True,
+        enable_cpr: bool = True,
     ) -> None:
-
         assert all(hasattr(stdout, a) for a in ("write", "flush"))
 
-        self._buffer: List[str] = []
+        self._buffer: list[str] = []
         self.stdout: TextIO = stdout
         self.default_color_depth = default_color_depth
         self._get_size = get_size
         self.term = term
         self.enable_bell = enable_bell
+        self.enable_cpr = enable_cpr
 
         # Cache for escape codes.
-        self._escape_code_caches: Dict[ColorDepth, _EscapeCodeCache] = {
+        self._escape_code_caches: dict[ColorDepth, _EscapeCodeCache] = {
             ColorDepth.DEPTH_1_BIT: _EscapeCodeCache(ColorDepth.DEPTH_1_BIT),
             ColorDepth.DEPTH_4_BIT: _EscapeCodeCache(ColorDepth.DEPTH_4_BIT),
             ColorDepth.DEPTH_8_BIT: _EscapeCodeCache(ColorDepth.DEPTH_8_BIT),
@@ -437,20 +439,25 @@ class Vt100_Output(Output):
         # default, we don't change them.)
         self._cursor_shape_changed = False
 
+        # Don't hide/show the cursor when this was already done.
+        # (`None` means that we don't know whether the cursor is visible or
+        # not.)
+        self._cursor_visible: bool | None = None
+
     @classmethod
     def from_pty(
         cls,
         stdout: TextIO,
-        term: Optional[str] = None,
-        default_color_depth: Optional[ColorDepth] = None,
+        term: str | None = None,
+        default_color_depth: ColorDepth | None = None,
         enable_bell: bool = True,
-    ) -> "Vt100_Output":
+    ) -> Vt100_Output:
         """
         Create an Output class from a pseudo terminal.
         (This will take the dimensions by reading the pseudo
         terminal attributes.)
         """
-        fd: Optional[int]
+        fd: int | None
         # Normally, this requires a real TTY device, but people instantiate
         # this class often during unit tests as well. For convenience, we print
         # an error message, use standard dimensions, and go on.
@@ -523,7 +530,7 @@ class Vt100_Output(Output):
             "eterm-color",
         ):  # Not supported by the Linux console.
             self.write_raw(
-                "\x1b]2;%s\x07" % title.replace("\x1b", "").replace("\x07", "")
+                "\x1b]2;{}\x07".format(title.replace("\x1b", "").replace("\x07", ""))
             )
 
     def clear_title(self) -> None:
@@ -531,7 +538,7 @@ class Vt100_Output(Output):
 
     def erase_screen(self) -> None:
         """
-        Erases the screen with the background colour and moves the cursor to
+        Erases the screen with the background color and moves the cursor to
         home.
         """
         self.write_raw("\x1b[2J")
@@ -652,10 +659,14 @@ class Vt100_Output(Output):
             self.write_raw("\x1b[%iD" % amount)
 
     def hide_cursor(self) -> None:
-        self.write_raw("\x1b[?25l")
+        if self._cursor_visible in (True, None):
+            self._cursor_visible = False
+            self.write_raw("\x1b[?25l")
 
     def show_cursor(self) -> None:
-        self.write_raw("\x1b[?12l\x1b[?25h")  # Stop blinking cursor and show.
+        if self._cursor_visible in (False, None):
+            self._cursor_visible = True
+            self.write_raw("\x1b[?12l\x1b[?25h")  # Stop blinking cursor and show.
 
     def set_cursor_shape(self, cursor_shape: CursorShape) -> None:
         if cursor_shape == CursorShape._NEVER_CHANGE:
@@ -703,6 +714,9 @@ class Vt100_Output(Output):
 
     @property
     def responds_to_cpr(self) -> bool:
+        if not self.enable_cpr:
+            return False
+
         # When the input is a tty, we assume that CPR is supported.
         # It's not when the input is piped from Pexpect.
         if os.environ.get("PROMPT_TOOLKIT_NO_CPR", "") == "1":
