@@ -1,9 +1,10 @@
 """
 The base classes for the styling.
 """
+
 from __future__ import annotations
 
-from abc import ABCMeta, abstractmethod, abstractproperty
+from abc import ABCMeta, abstractmethod
 from typing import Callable, Hashable, NamedTuple
 
 __all__ = [
@@ -28,6 +29,7 @@ class Attrs(NamedTuple):
     blink: bool | None
     reverse: bool | None
     hidden: bool | None
+    dim: bool | None
 
 
 """
@@ -40,6 +42,7 @@ class Attrs(NamedTuple):
 :param blink: Boolean
 :param reverse: Boolean
 :param hidden: Boolean
+:param dim: Boolean
 """
 
 #: The default `Attrs`.
@@ -53,6 +56,7 @@ DEFAULT_ATTRS = Attrs(
     blink=False,
     reverse=False,
     hidden=False,
+    dim=False,
 )
 
 
@@ -122,7 +126,8 @@ class BaseStyle(metaclass=ABCMeta):
         :param default: `Attrs` to be used if no styling was defined.
         """
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def style_rules(self) -> list[tuple[str, str]]:
         """
         The list of style rules, used to create this style.
